# Jason Giglio <jgiglio@netmar.com>
# Released under the GPL v2 or later.
#
# NO WARRANTY
# BECAUSE THE PROGRAM IS LICENSED FREE OF CHARGE, THERE IS NO WARRANTY
# FOR THE PROGRAM, TO THE EXTENT PERMITTED BY APPLICABLE LAW.  EXCEPT WHEN
# OTHERWISE STATED IN WRITING THE COPYRIGHT HOLDERS AND/OR OTHER PARTIES
# PROVIDE THE PROGRAM "AS IS" WITHOUT WARRANTY OF ANY KIND, EITHER EXPRESSED
# OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF
# MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE.  THE ENTIRE RISK AS
# TO THE QUALITY AND PERFORMANCE OF THE PROGRAM IS WITH YOU.  SHOULD THE
# PROGRAM PROVE DEFECTIVE, YOU ASSUME THE COST OF ALL NECESSARY SERVICING,
# REPAIR OR CORRECTION.
# 
# WARNING! This script makes system() type calls to many external 
# commands. I have made every attempt to protect the system from 
# a malicious user, but such calls are always dangerous.
#
#
# Usage:
# !ampacity - Uses functions derived from curve-fitted data to 
#	judge copper wire ampacity
# !units - Calls the units program, only two args taken
# !math - Calls the bc program, arbitrary length args
#         and also a factorial function is built in f(x)
# !htam - Calls the dc program, arbitrary length args, RPN calculator
# !wire - Gives info on wire gauge in AWG
# !traceohms - Gives ohms of PCB trace
# !traceamp -  tells the required tracewidth to carry supplied amps
# !awg - Takes size in mils, converts to AWG
# 
# !help - sends a usage message
#
# 2.3	Jan 22, 2006
#	- Added ampacity trigger, works based on curve fitted functions
#	
# 2.2	Apr 3, 2005
#	- Added feature to allow only some channels to use triggers
#	- Added ability for people to /msg you triggers without you seeing
#		the query windows
#	- Added !awg, traceohms, traceamp
#
# 2.1   Mar 6, 2005
#	- made units more flexible
#	- added help
#	- added wire
#	- made trigger char user-definiable
#	- Added license text and warnings
#
# 2.0   Feb 10, 2005
#	- added !math to call bc
#	- added !htam to call dc
#
# 1.0	Jan 22, 2005
#	- Initial release
#
 
#CONFIG VARIABLES
trigger = '!'
channels={'##electronics':1, '#test':1, '#shellglitch':1, '#robotics':1}
#END CONFIG

import xchat
import os
import string

__module_name__ = "engineering"
__module_version__ = "2.0"
__module_description__ = "Xchat Engineering bot script"


trans = string.maketrans("","")
usage={}

usage["units"]="Usage: " + trigger + "units fromunit tounit e.g. 1km mile"
def units(nick, args):
	args = args.translate(trans, ";'&`*?~<>^[]{}$'").replace("define","").lstrip()
	quotedwords = " ".join(["'%s'" % word for word in args.split()])
	cmdline="units -v " + quotedwords
	fd_units = os.popen(cmdline,"r")
	 
    	chanmesg = "say " + nick + ": " + fd_units.readline().strip().replace("inf","breezy").lstrip()
    	xchat.command(chanmesg)
	fd_units.close()

usage["awg"]="Usage: " + trigger + "awg <mils>, convert mils to awg"
def awg(nick, args):
	args=args.replace("'", "")
        args=args.replace(";", "")
        mils=str(args)
	params="'%s mils' 'wiregauge'" % mils
	cmdline="units " + params 
	fd_units = os.popen(cmdline,"r")
        awg = fd_units.readline().strip()
	chanmesg = "say " + nick + ": " + mils + " mils = "+ awg +" AWG"
	xchat.command(chanmesg)
        fd_units.close()

usage["wire"]="Usage: " + trigger + "wire <AWG>, gives stats for copper wire of <AWG> size"
def wire(nick, args):
	args=args.replace("'", "")
        args=args.replace(";", "")
	gauge=str(args)
        params="'wiregauge(%s)'" % gauge
        cmdline="units " + params + " mil"
        fd_units = os.popen(cmdline,"r")
        mil = fd_units.readline().strip()[1:]
	cmil = float(mil)**2
        chanmesg = "say " + nick + ": "+ gauge + "AWG ("+ str(round(float(cmil),2)) +"cmils) @ " 
	chanmesg = chanmesg + "TPI: " + str(round(1000/float(mil),3)) + "turns "
	chanmesg = chanmesg + "Ohm/Ft: " + str(round(10.37*(1/float(cmil)),6))
	chanmesg = chanmesg + " Diameter: " + str(mil) + " mils"
        xchat.command(chanmesg)
        fd_units.close()

usage["ampacity"]="Usage: " + trigger + "ampacity <AWG>, gives ampacity for copper wire of <AWG> size"
def ampacity(nick, args):
	args=args.replace("'", "")
        args=args.replace(";", "")
	gauge=str(args)
        params="'wiregauge(%s)'" % gauge
        cmdline="units " + params + " in"
        fd_units = os.popen(cmdline,"r")
        diameter = fd_units.readline().strip()[1:]
	
	if (float(diameter)<=0.026):
		freeampacity = 0.31648 * 2.71828183**(float(diameter)/0.00711)-0.42085
		enclosedampacity=0.07005 * 2.71828183**(float(diameter)/0.00590)+0.13427
	else:
		freeampacity = 500.01941 * 2.71828183**(float(diameter)/0.79238)-511.15794
		enclosedampacity= 287.19085 * 2.71828183**(float(diameter)/0.76306)-294.40413
       
        #sanity checks
	if (freeampacity < 0):
		freeampacity=0
	if (enclosedampacity > freeampacity):
		enclosedampacity=0
		
	cmil = (float(diameter)*1000)**2
	
	chanmesg = "say " + nick + ": "+ gauge + "AWG Cu Wire Free-air: "+ str(round(freeampacity,3))  +"A, " 
	chanmesg = chanmesg + "Enclosed: "+ str(round(enclosedampacity,3))  +"A, " 
	chanmesg = chanmesg + "700CM/A: " + str(round(float(cmil)/700,3)) + "A "
        xchat.command(chanmesg)
        fd_units.close()



usage["traceohms"]="Usage: " + trigger + "traceohms <mil> <oz>, tells ohms of trace <mils> wide on <oz> weight copper clad "
def traceohms(nick, args):
	args=args.translate(trans, "';").lstrip()
	arglist=args.split()
        mils=float(arglist[0])
        oz=float(arglist[1])
	ohms=(10.37*(1/(mils*1.755*oz))/12)
	chanmesg = "say " + nick + ": "
	chanmesg = chanmesg + str(mils) + " mil wide trace on "+ str(oz) + "oz copper clad: "
        chanmesg = chanmesg + str(round(ohms,6)) + " Ohms/Inch"
        xchat.command(chanmesg)


usage["traceamp"]="Usage: " + trigger + "traceamp <amps> <ounces> <tempRiseinC>, tells the required tracewidth for copper clad of weight <ounces> to carry <amps> with a max temperature rise of <temprise> over ambient"
def traceamp(nick, args):
	args=args.translate(trans, "';").lstrip()
	arglist=args.split()
	amps=float(arglist[0])
	oz=float(arglist[1])
	temprise=float(arglist[2])
	internalarea=(amps/(0.0150*(temprise)**0.5453))**(1/0.7349)	
	internalwidth=internalarea/(oz*1.378)
	surfacearea=(amps/(0.0647*(temprise)**0.4281))**(1/0.6732)	
	surfacewidth=surfacearea/(oz*1.378)
	chanmesg = "say " + nick 
	chanmesg=chanmesg + ": Internal: " + str(round(internalwidth)) + "mils,"
	chanmesg=chanmesg + " Surface: " + str(round(surfacewidth)) + "mils" 
	xchat.command(chanmesg)
	


usage["math"]="Usage: " + trigger + "math <mathstuff> e.g. 1+1"
def bc(nick, args):
	args = args.translate(trans, "'").replace("define","").lstrip()
	fullargs = "define f (x) {if (x <= 1) return (1); return (f(x-1) * x);} " + args
        fd_bc = os.popen("echo '%s' | bc -l" % fullargs)
	result=fd_bc.read().strip()
	if len(result) > 50:
		result="Truncated: " + result
		result=result[0:50]
        chanmesg = "say " + nick + ": " + args + " = " + result
        xchat.command(chanmesg)
	fd_bc.close()

usage["htam"]="Usage: " + trigger + "htam <RPNmath> e.g. 1 1 + p"
def dc(nick, args):
	args = args.translate(trans, "'").lstrip()
        fd_dc = os.popen("echo '%s' | dc" % args)
	result=fd_dc.read().strip()
	if len(result) > 50:
		result="Truncated: " + result
		result=result[0:50]
        chanmesg = "say " + nick + ": " + args + " = " + result
        xchat.command(chanmesg)
	fd_dc.close()

def help(nick, args):
	helpkeys=usage.keys()	
	for x in helpkeys: 
  		xchat.command("msg "+ nick + ' ' + usage[x])


def chanmsg_cb(word, word_eol, userdata): 
	channel=xchat.get_info('channel')
	#if the channel is not in the list, and isn't a query
	if not channels.has_key(channel) and channel.startswith("#"):
		return xchat.EAT_NONE

	args = ' '.join(word[1].split()[1:])
	
	if word[1].startswith(trigger):
		triggered=1
	else:
		triggered=0

	if word[1].startswith(trigger + "units"):
		arglen=len(args.split())
                if arglen == 0:
                        xchat.command("say "+ word[0] + ": " + usage["units"])
                else:
			units(word[0], args)

	if word[1].startswith(trigger + "math"):
	        arglen=len(args.split())
                if arglen == 0:
                        xchat.command("say "+ word[0] + ": " + usage["math"])
                else:
        		bc(word[0], args)

	if word[1].startswith(trigger + "htam"):
		arglen=len(args.split())
                if arglen == 0:
                        xchat.command("say "+ word[0] + ": " + usage["htam"])
                else:
			dc(word[0], args)

        if word[1].startswith(trigger + "awg"):
                arglen=len(args.split())
                if arglen != 1:
                        xchat.command("say "+ word[0] + ": " + usage["awg"])
                else:
                        awg(word[0], args)

	if word[1].startswith(trigger + "wire"):
		arglen=len(args.split())
		if arglen != 1:
			xchat.command("say "+ word[0] + ": " + usage["wire"])
		else:
			wire(word[0], args)

	if word[1].startswith(trigger + "traceamp"):
		arglen=len(args.split())
		if arglen != 3:
			xchat.command("say "+ word[0] + ": " + usage["traceamp"])
		else:
                	traceamp(word[0], args)

	if word[1].startswith(trigger + "traceohms"):
		arglen=len(args.split())
                if arglen != 2:
                        xchat.command("say "+ word[0] + ": " + usage["traceohms"])
                else:
			traceohms(word[0], args)

        if word[1].startswith(trigger + "ampacity"):
		arglen=len(args.split())
		if arglen != 1:
			xchat.command("say "+ word[0] + ": " + usage["ampacity"])
		else:
			ampacity(word[0], args)


	if word[1].startswith(trigger + "help"):
		help(word[0],args)


	#this part closes queries that use triggers
	#this prevents you from getting spammed with 
	#a million query windows when people use the triggers
	if not channel.startswith('#') and triggered:
		channelcontext=xchat.get_context()
		channelcontext.command("close")
		return xchat.EAT_ALL
	return xchat.EAT_NONE



	
xchat.hook_print("Channel Message",chanmsg_cb) 
xchat.hook_print("Private Message to Dialog",chanmsg_cb) 
xchat.hook_print("Your Message",chanmsg_cb) 

print "Engineering script loaded."
